package com.fluttercandies.flutter_bdface_collect;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;

import com.baidu.idl.face.platform.FaceStatusNewEnum;
import com.baidu.idl.face.platform.model.ImageInfo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FlutterDetectActivity  extends FaceDetectActivity implements
        TimeoutDialog.OnTimeoutDialogClickListener {

    private TimeoutDialog mTimeoutDialog;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onDetectCompletion(FaceStatusNewEnum status, String message,
                                   HashMap<String, ImageInfo> base64ImageCropMap,
                                   HashMap<String, ImageInfo> base64ImageSrcMap) {
        super.onDetectCompletion(status, message, base64ImageCropMap, base64ImageSrcMap);
        if (status == FaceStatusNewEnum.OK && mIsCompletion) {
            // 获取最优图片
            getBestImage(base64ImageCropMap, base64ImageSrcMap);
        } else if (status == FaceStatusNewEnum.DetectRemindCodeTimeout) {
            if (mViewBg != null) {
                mViewBg.setVisibility(View.VISIBLE);
            }
            showMessageDialog();
        }
    }

    /**
     * 获取最优图片
     * @param imageCropMap 抠图集合
     * @param imageSrcMap  原图集合
     */
    private void getBestImage(HashMap<String, ImageInfo> imageCropMap, HashMap<String, ImageInfo> imageSrcMap) {
        String imageSrcBase64 = "";
        String imageCropBase64 = "";

        // 将抠图集合中的图片按照质量降序排序，最终选取质量最优的一张抠图图片
        if (imageCropMap != null && imageCropMap.size() > 0) {
            List<Map.Entry<String, ImageInfo>> list1 = new ArrayList<>(imageCropMap.entrySet());
            Collections.sort(list1, (o1, o2) -> {
                String[] key1 = o1.getKey().split("_");
                String score1 = key1[2];
                String[] key2 = o2.getKey().split("_");
                String score2 = key2[2];
                return Float.valueOf(score2).compareTo(Float.valueOf(score1));
            });
            //获取抠图中的加密或非加密的base64
            int secType = mFaceConfig.getSecType();
            if (secType == 0) {
                imageCropBase64 = list1.get(0).getValue().getBase64();
            } else {
                imageCropBase64 = list1.get(0).getValue().getSecBase64();
            }
        }
        // 将原图集合中的图片按照质量降序排序，最终选取质量最优的一张原图图片
        if (imageSrcMap != null && imageSrcMap.size() > 0) {
            List<Map.Entry<String, ImageInfo>> list2 = new ArrayList<>(imageSrcMap.entrySet());
            Collections.sort(list2, (o1, o2) -> {
                String[] key1 = o1.getKey().split("_");
                String score1 = key1[2];
                String[] key2 = o2.getKey().split("_");
                String score2 = key2[2];
                return Float.valueOf(score2).compareTo(Float.valueOf(score1));
            });
            // 获取原图中的加密或非加密的base64
            int secType = mFaceConfig.getSecType();
            if (secType == 0) {
                imageSrcBase64 = list2.get(0).getValue().getBase64();
            } else {
                imageSrcBase64 = list2.get(0).getValue().getSecBase64();
            }
        }

        Intent intent = new Intent();//数据是使用Intent返回
        intent.putExtra("imageCropBase64", imageCropBase64);//把返回数据存入Intent
        intent.putExtra("imageSrcBase64", imageSrcBase64);//把返回数据存入Intent
        this.setResult(FlutterBdFaceSetting.COLLECT_OK_CODE,intent);//设置返回数据
        this.finish();//关闭Activity
    }

    private void showMessageDialog() {
        mTimeoutDialog = new TimeoutDialog(this);
        mTimeoutDialog.setDialogListener(this);
        mTimeoutDialog.setCanceledOnTouchOutside(false);
        mTimeoutDialog.setCancelable(false);
        mTimeoutDialog.show();
        mHasShownTimeoutDialog = true;
        onPause();
    }

    @Override
    public void finish() {
        super.finish();
    }

    @Override
    public void onRecollect() {
        if (mTimeoutDialog != null) {
            mTimeoutDialog.dismiss();
            mHasShownTimeoutDialog = false;
        }
        if (mViewBg != null) {
            mViewBg.setVisibility(View.GONE);
        }
        onResume();
    }

    @Override
    public void onReturn() {
        if (mTimeoutDialog != null) {
            mTimeoutDialog.dismiss();
            mHasShownTimeoutDialog = false;
        }
        finish();
    }
}
